// TTGO_Tnc v1.00 : Programme TNC LoRa pour M5Stack.
// Ecrit par Frank LOBA (www.f4hdb.fr).
// Date création : 24/03/2023.
// Dernière mise à jour : 24/03/2023.


/**** INCLUSION DES FICHIERS EN-TETE ****/
#include <SPI.h>
#include <LoRa.h>

#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#include <M5Kiss.h>


/**** DEFINITION DES CONSTANTES ****/
#define OLED_SDA 21
#define OLED_SCL 22 
#define OLED_RST 16

#define OLED_SCREEN_WIDTH   128
#define OLED_SCREEN_HEIGHT  64

#define OLED_ADRESS 0x3C


#define LORA_SCK  5
#define LORA_SS   18
#define LORA_MISO 19  
#define LORA_MOSI 27
#define LORA_RST  23
#define LORA_DIO0 26

#define LORA_FREQUENCE         433775000L
#define LORA_SPREADING_FACTOR  12
#define LORA_SIGNAL_BANDWIDTH  125000L
#define LORA_CODING_RATE_4     5
#define LORA_SYNC_WORD         0x12

#define LORA_START_BYTE_1 '<'
#define LORA_START_BYTE_2 0xFF
#define LORA_START_BYTE_3 0x01


/**** DEFINITION DES VARIABLES GLOBALES ****/
Adafruit_SSD1306 Display(OLED_SCREEN_WIDTH, OLED_SCREEN_HEIGHT, &Wire, OLED_RST);

String  TamponTNC2;
String  TamponAX25;
String  TamponKISS;


/**** DEFINITION DE LA FONCTION D'INITIALISATION ****/
void setup()
{
  // Initialisation de l'interface série (USB).
  Serial.begin(115200);

  // Initialisation de l'écran OLED.
  Wire.begin(OLED_SDA, OLED_SCL);
  
  if(!Display.begin(SSD1306_SWITCHCAPVCC, OLED_ADRESS, false, false))
  {
    // Ecriture d'un message d'erreur dans le port série en cas d'échec.
    Serial.println("Echec initialisation ecran");
    
    while(1);
  }

  // Effacement de l'écran, définition de la couleur et de la taille du texte.
  Display.clearDisplay();
  Display.setTextColor(WHITE);
  Display.setTextSize(1);

  // Affectation des broches pour LoRa.
  SPI.begin(LORA_SCK, LORA_MISO, LORA_MOSI, LORA_SS);
  LoRa.setPins(LORA_SS, LORA_RST, LORA_DIO0);


  // Défintion de la fréquence et démarrage du LoRa.
  if(!LoRa.begin(LORA_FREQUENCE))
  {
    // Ecriture d'un message d'erreur dans le port série en cas d'échec.
    Serial.println("Echec demarrage LoRa");

    // Affichage d'un message à l'écran en cas d'échec.
    Display.setCursor(0,0);
    Display.println("Echec demarrage LoRa");
    Display.display();
    
    while (1);
  }

  // Définition des paramètres système pour LoRa.
  LoRa.setSpreadingFactor(LORA_SPREADING_FACTOR);
  LoRa.setSignalBandwidth(LORA_SIGNAL_BANDWIDTH);
  LoRa.setCodingRate4(LORA_CODING_RATE_4);
  LoRa.enableCrc();
  LoRa.setSyncWord(LORA_SYNC_WORD);

  // Ecriture d'un message dans le port série en cas de succès.
  Serial.println("LoRa OK");

  // Affichage d'un message à l'écran en cas de succès.
  Display.setCursor(0,0);
  Display.println("LoRa OK");
  Display.display();  
}


/**** DEFINITION DE LA FONCTION D'EXECUTION ****/
void loop()
{
  //**** Partie réception radio.

  // Détection d'un message reçu par radio.
  int PacketSize = LoRa.parsePacket();
 
  if (PacketSize)
  {
    // Initialisation du tampon de la trame TNC2.
    TamponTNC2 = "";
    
    // Effacement de l'écran.
    Display.clearDisplay();

    // Positionnement du curseur.
    Display.setCursor(0,0);

    // Affichage de "RX" à l'écran.
    Display.println("**** RX ****");
    Display.display();

    // Détection de l'en-tête LoRa.
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_1)
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_2)
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_3)
    // Lecture du paquet au format TNC2.
    while(LoRa.available())
    TamponTNC2 += (char)LoRa.read();

    // Affichage de la trame au format TNC2 à l'écran.
    Display.println(TamponTNC2);
    Display.display();

    // Conversion du format TNC2 au format AX25.
    TamponAX25 = TNC2toAX25(TamponTNC2);

    // La conversion a réussi.
    if(TamponAX25.length() > 0)
    {
      // Encodage de la trame AX25 avec le protocole KISS.
      TamponKISS = EncoderKISS(TamponAX25);
  
      // Ecriture dans le port série.
      Serial.print(TamponKISS);
    }
    // La conversion a échoué.
    else
    {
      Display.println("Echec conversion TNC2 vers AX25");
      Display.display();
    }
      
    // Affichage du RSSI à l'écran.
    Display.print("RSSI : ");
    Display.println(LoRa.packetRssi());
    Display.display();
  }


  //**** Partie émission radio.

  // Détection d'un message dans le port série.
  if(Serial.available())
  {
    // Initialisation du Tampon KISS.
    TamponKISS = "";

    // Effacement de l'écran.
    Display.clearDisplay();

    // Positionnement du curseur.
    Display.setCursor(0,0);

    // Affichage de "TX" à l'écran.
    Display.println("**** TX ****");
    Display.display();

    // Lecture de la trame KISS dans le port série.
    while(Serial.available())
    TamponKISS += (char) Serial.read();

    // Décodage de la trame KISS.
    TamponAX25 = DecoderKISS(TamponKISS);

    // Le décodage a réussi.
    if(TamponAX25.length() > 0)
    {
      // Conversion du format AX25 au format TNC2.
      TamponTNC2 = AX25toTNC2(TamponAX25);

      // La conversion a réussi.
      if(TamponTNC2.length() > 0)
      {
        // Commencement de l'émission LoRa.
        LoRa.beginPacket();
            
        // Emission de l'en-tête LoRa.
        LoRa.write(LORA_START_BYTE_1);
        LoRa.write(LORA_START_BYTE_2);
        LoRa.write(LORA_START_BYTE_3);
            
        // Emission de la trame TNC2 en LoRa.
        LoRa.write((const uint8_t *)TamponTNC2.c_str(), TamponTNC2.length());
            
        // Fin de l'émission LoRa.
        LoRa.endPacket();

        // Affichage de la trame au format TNC2 à l'écran.
        Display.println(TamponTNC2);     
        Display.display();
      }
      // La conversion a échoué.
      else
      {
        Display.println("Echec conversion AX25 vers TNC2"); 
        Display.display();
      }
    }
    // Le décodage a échoué.
    else
    {
      Display.println("Echec décodage KISS"); 
      Display.display();
    }
  }

  delay(20);
}
